% Adaptive Finite Element Method for Boundary Value Problem
clear; clc;

% Define problem parameters
f = @(x) sin(pi*x); % Example source function, replace as needed
epsilon = 1/100;     % Stopping criterion
alpha = 0.7;        % Refinement parameter

% Initialization
T = [0, 0.5, 1];    % Initial mesh
max_iter = 100;     % Maximum number of iterations
i = 0;              % Iteration counter

fig = figure();
while i < max_iter
    i = i + 1;
    
    % Step (a): Solve the discrete problem
    [u, K_global] = solve_problem(T, f);
    
    % Step (b): Estimate error
    [eta, eta_max] = estimate_error(T, u, f);
    figure(fig)
    subplot(1,2,2)
        hold on;
        plot(T(1:(end-1))+ 0.5*diff(T),log(eta),'.')
        hold on;
        plot(T(1:(end)),log(alpha*eta_max + 0*T(1:(end))))
    
    % Check stopping criterion
    if eta_max <= epsilon
        fprintf('Converged after %d iterations.\n', i);
        break;
    end
    
    % Step (c): Mark intervals for refinement
    marked = find(eta >= alpha * eta_max);
    
    % Step (d): Refine mesh
    T = refine_mesh(T, marked);
end

% Plot the solution
figure(fig);
x = linspace(0, 1, 100);
subplot(1,2,1)
    plot(x, exact_solution(x), 'r-', 'DisplayName', 'Exact Solution');
    hold on;
    plot(T, u, 'bo-', 'DisplayName', 'Numerical Solution');
    legend;
    xlabel('x','Interpreter','Latex');
    ylabel('u(x)','Interpreter','Latex');
    title('Adaptive FEM Solution','Interpreter','Latex');
subplot(1,2,2)
    title('Error indicator','Interpreter','Latex')
    xlabel('$x$','Interpreter','Latex');
    ylabel('$\log(\eta)$','Interpreter','Latex');

function u_exact = exact_solution(x)
    % Exact solution for verification
    u_exact = sin(pi*x)/pi^2;
end