% Adaptive Finite Element Method for Boundary Value Problem
clear; clc;

% Define problem parameters
f = @(x) x.^(-1/3); % Example source function, replace as needed
epsilon = 1/100;     % Stopping criterion
alpha = 0.7;        % Refinement parameter

% Initialization
T = [0, 0.5, 1];    % Initial mesh
max_iter = 100;     % Maximum number of iterations
i = 0;              % Iteration counter

% Exact solution
exact_solution = @(x) -9/10*x.*(x.^(2/3)-1); % Exact solution for verification
exact_solution_prime = @(x) 9/10 - (3*x.^(2/3))/2; % Deriviative for exact solution for verification

fig = figure();
while i < max_iter
    i = i + 1;
    
    % Step (a): Solve the discrete problem
    [u, K_global] = solve_problem(T, f);
    
    % Step (b): Estimate error
    [eta, eta_max] = estimate_error(T, u, f);

    % Estimate true H^1 error
    [H1_error, element_errors] = true_error(exact_solution, exact_solution_prime, T, u);

    % Plotting errors
        figure(fig)
        subplot(2,2,2)
            hold on;
            plot(T(1:(end-1))+ 0.5*diff(T),log(eta),'.')
            plot(T(1:(end)),log(alpha*eta_max + 0*T(1:(end))))
        
        subplot(2,2,3)
            hold on;
            plot(T(1:(end-1))+ 0.5*diff(T),log(element_errors),'.')

        subplot(2,2,4)
            hold on;
            plot(T,i + 0*T(1:(end)),'x')
    
    % Check stopping criterion
    if eta_max <= epsilon
        fprintf('Converged after %d iterations.\n', i);
        break;
    end
    
    % Step (c): Mark intervals for refinement
    marked = find(eta >= alpha * eta_max);
    
    % Step (d): Refine mesh
    T = refine_mesh(T, marked);
end

%% Plot the solution
figure(fig);
x = linspace(0, 1, 100);

subplot(2,2,1)
    plot(x, exact_solution(x), 'r-', 'DisplayName', 'Exact Solution');
    hold on;
    plot(T, u, 'bo-', 'DisplayName', 'Numerical Solution');
    legend;
    xlabel('x');
    ylabel('u(x)');
    title('Adaptive FEM Solution and true solution','Interpreter','Latex');
subplot(2,2,2)
    title('Error indicator','Interpreter','Latex')
    xlabel('$x$','Interpreter','Latex');
    ylabel('$\log(\eta)$','Interpreter','Latex');
subplot(2,2,3)
    title('$H^1$ error','Interpreter','Latex')
    xlabel('$x$','Interpreter','Latex');
    ylabel('$\log(|| u-u_S ||_{H^1(K_i)})$','Interpreter','Latex');
subplot(2,2,4)
    title('Mesh refinements','Interpreter','Latex')
    xlabel('$x$','Interpreter','Latex');
    ylabel('Iteration','Interpreter','Latex');


