function [H1_error, element_errors] = true_error(u_true, u_true_prime, T, u_h)
    % TRUE_ERROR Computes the H^1 error of the finite element approximation
    % on the entire domain and on each mesh element.
    %
    % Inputs:
    %   u_true       - Function handle for the true solution u(x)
    %   u_true_prime - Function handle for the derivative of the true solution u'(x)
    %   T            - Finite element mesh (vector of nodal coordinates)
    %   u_h          - Approximate solution (vector of nodal values)
    %
    % Outputs:
    %   H1_error     - Total H^1 error over the entire domain
    %   element_errors - Array with H^1 error on each mesh element

    n = length(T) - 1; % Number of elements
    element_errors = zeros(n, 1); % Initialize element errors
    H1_error_squared = 0; % Initialize squared total error

    for k = 1:n
        % Element boundaries
        x1 = T(k);
        x2 = T(k+1);
        h = x2 - x1;

        % Nodal values of the approximation
        u1_h = u_h(k);
        u2_h = u_h(k+1);

        % Linear shape function derivatives
        u_h_prime = (u2_h - u1_h) / h;

        % Define local integrands
        f_L2 = @(x) (u_true(x) - ((u2_h - u1_h) / h * (x - x1) + u1_h)).^2;
        f_H1 = @(x) (u_true_prime(x) - u_h_prime).^2;

        % Compute the integrals using midpoint rule or numerical quadrature
        integral_L2 = integral(f_L2, x1, x2); % Use 10 subintervals
        integral_H1 = integral(f_H1, x1, x2);

        % Compute the error on this element
        element_error_squared = integral_L2 + integral_H1;
        element_errors(k) = sqrt(element_error_squared);

        % Add to the total squared error
        H1_error_squared = H1_error_squared + element_error_squared;
    end

    % Final H^1 error
    H1_error = sqrt(H1_error_squared);
end